@@ -12,12 +12,22 @@
 # limitations under the License.
 
 import functools
+import os
+import signal
+from six.moves import BaseHTTPServer
+from six.moves import http_client as http
+from stevedore import extension
+import tempfile
+import tenacity
 
+from tempest import config
 from tempest.lib.common.utils import data_utils
 from tempest.lib import exceptions
 
 from senlin_tempest_plugin.common import constants
 
+CONF = config.CONF
+
 
 def api_microversion(api_microversion):
     """Decorator used to specify api_microversion for test."""
@@ -32,9 +42,26 @@ def api_microversion(api_microversion):
     return decorator
 
 
-def prepare_and_cleanup_for_nova_server(base):
+def is_policy_supported(policy_name_and_version):
+    """Check if specified policy is supported
+
+    :param policy_name_and_version: Combined string of policy name and version.
+    E.g. senlin.policy.scaling-1.0
+    :returns: True if policy_name_and_version is supported. False otherwise.
+    """
+    mgr = extension.ExtensionManager(
+        namespace='senlin.policies',
+        invoke_on_load=False)
+    return policy_name_and_version in mgr.names()
+
+
+def prepare_and_cleanup_for_nova_server(base, cidr, spec=None):
     keypair_name = create_a_keypair(base, is_admin_manager=False)
-    base.spec = constants.spec_nova_server
+    if spec is None:
+        base.spec = constants.spec_nova_server
+    else:
+        base.spec = spec
+
     base.spec['properties']['key_name'] = keypair_name
     base.addCleanup(delete_a_keypair, base, keypair_name,
                     is_admin_manager=False)
@@ -43,10 +70,20 @@ def prepare_and_cleanup_for_nova_server(base):
     network_id = create_a_network(base, name=n_name)
     base.addCleanup(delete_a_network, base, network_id)
 
-    subnet_id = create_a_subnet(base, network_id, "192.168.199.0/24")
+    subnet_id = create_a_subnet(base, network_id, cidr)
     base.addCleanup(delete_a_subnet, base, subnet_id)
 
 
+def create_spec_from_config():
+    """Utility function that creates a spec object from tempest config"""
+    spec = constants.spec_nova_server
+
+    spec['properties']['flavor'] = CONF.compute.flavor_ref
+    spec['properties']['image'] = CONF.compute.image_ref
+
+    return spec
+
+
 def create_a_profile(base, spec=None, name=None, metadata=None):
     """Utility function that generates a Senlin profile."""
 
@@ -56,11 +93,13 @@ def create_a_profile(base, spec=None, name=None, metadata=None):
     if name is None:
         name = data_utils.rand_name("tempest-created-profile")
 
+    if metadata:
+        spec['properties']['metadata'] = metadata
+
     params = {
         'profile': {
             'name': name,
             'spec': spec,
-            'metadata': metadata,
         }
     }
     res = base.client.create_obj('profiles', params)
@@ -146,14 +185,29 @@ def list_clusters(base):
     return res['body']
 
 
+def _return_last_value(retry_state):
+    return retry_state.outcome.result()
+
+
+@tenacity.retry(
+    retry=(tenacity.retry_if_exception_type(exceptions.Conflict) |
+           tenacity.retry_if_result(lambda x: x is False)),
+    wait=tenacity.wait_fixed(2),
+    retry_error_callback=_return_last_value,
+    stop=tenacity.stop_after_attempt(5)
+)
 def delete_a_cluster(base, cluster_id, wait_timeout=None):
     """Utility function that deletes a Senlin cluster."""
     res = base.client.delete_obj('clusters', cluster_id)
     action_id = res['location'].split('/actions/')[1]
-    base.client.wait_for_status('actions', action_id, 'SUCCEEDED',
-                                wait_timeout)
+
+    action = base.client.wait_for_status(
+        'actions', action_id, ['SUCCEEDED', 'FAILED'], wait_timeout)
+    if action['body']['status'] == 'FAILED':
+        return False
+
     base.client.wait_for_delete("clusters", cluster_id, wait_timeout)
-    return
+    return True
 
 
 def create_a_node(base, profile_id, cluster_id=None, metadata=None,
@@ -284,6 +338,13 @@ def cluster_attach_policy(base, cluster_id, policy_id,
     return res['body']['status_reason']
 
 
+@tenacity.retry(
+    retry=(tenacity.retry_if_exception_type(exceptions.Conflict) |
+           tenacity.retry_if_result(lambda x: x is False)),
+    wait=tenacity.wait_fixed(2),
+    retry_error_callback=_return_last_value,
+    stop=tenacity.stop_after_attempt(5)
+)
 def cluster_detach_policy(base, cluster_id, policy_id,
                           expected_status='SUCCEEDED', wait_timeout=None):
     """Utility function that detach a policy from cluster."""
@@ -295,8 +356,12 @@ def cluster_detach_policy(base, cluster_id, policy_id,
     }
     res = base.client.trigger_action('clusters', cluster_id, params=params)
     action_id = res['location'].split('/actions/')[1]
-    res = base.client.wait_for_status('actions', action_id, expected_status,
-                                      wait_timeout)
+
+    res = base.client.wait_for_status(
+        'actions', action_id, ['SUCCEEDED', 'FAILED'], wait_timeout)
+    if res['body']['status'] == 'FAILED':
+        return False
+
     return res['body']['status_reason']
 
 
@@ -507,7 +572,8 @@ def create_a_network(base, name=None):
     return body['body']['id']
 
 
-def delete_a_network(base, network_id, ignore_missing=False):
+def delete_a_network(base, network_id, ignore_missing=False,
+                     wait_timeout=None):
     """Utility function that deletes a Neutron network."""
 
     res = base.network_client.delete_obj('networks', network_id)
@@ -516,6 +582,8 @@ def delete_a_network(base, network_id, ignore_missing=False):
             return
         raise exceptions.NotFound()
 
+    base.network_client.wait_for_delete('networks', network_id, wait_timeout)
+
 
 def create_a_subnet(base, network_id, cidr, ip_version=4, name=None):
     """Utility function that creates a Neutron subnet"""
@@ -536,7 +604,7 @@ def create_a_subnet(base, network_id, cidr, ip_version=4, name=None):
     return body['body']['id']
 
 
-def delete_a_subnet(base, subnet_id, ignore_missing=False):
+def delete_a_subnet(base, subnet_id, ignore_missing=False, wait_timeout=None):
     """Utility function that deletes a Neutron subnet."""
 
     res = base.network_client.delete_obj('subnets', subnet_id)
@@ -545,6 +613,8 @@ def delete_a_subnet(base, subnet_id, ignore_missing=False):
             return
         raise exceptions.NotFound()
 
+    base.network_client.wait_for_delete('subnets', subnet_id, wait_timeout)
+
 
 def create_queue(base, queue_name):
     """Utility function that creates Zaqar queue."""
@@ -582,3 +652,48 @@ def post_messages(base, queue_name, messages):
     if res['status'] != 201:
         msg = 'Failed in posting messages to Zaqar queue %s' % queue_name
         raise Exception(msg)
+
+
+def start_http_server(port=5050):
+    def _get_http_handler_class(filename):
+        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+
+            def do_GET(self):
+                data = b'healthy\n'
+                self.send_response(http.OK)
+                self.send_header('Content-Length', str(len(data)))
+                self.end_headers()
+                self.wfile.write(data)
+                return
+
+            def log_message(self, fmt, *args):
+                msg = fmt % args
+                with open(filename, 'a+') as tmp:
+                    tmp.write("%s\n" % msg)
+                return
+
+        return StaticHTTPRequestHandler
+
+    server_address = ('127.0.0.1', port)
+    new_file, filename = tempfile.mkstemp()
+    handler_class = _get_http_handler_class(filename)
+    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
+
+    pid = os.fork()
+    if pid == 0:
+        httpd.serve_forever()
+    else:
+        return pid, filename
+
+
+def terminate_http_server(pid, filename):
+    os.kill(pid, signal.SIGKILL)
+
+    if not os.path.isfile(filename):
+        return ''
+
+    with open(filename, 'r') as f:
+        contents = f.read()
+
+    os.remove(filename)
+    return contents
